library(ape)
# read the trees
tree <- read.nexus("../data/proc.data/pruned.trees.no.poly.nex")
# read the data
chrom <- read.csv("../data/proc.data/pruned.chrom.no.poly.csv")

library(chromePlus)
####
#### state one is mixed karyotype

# check the range of chromosome numbers present
range(chrom[,2])

# slim the data to include only the desired data
chrom <- chrom[, c(1,2,6)]

### these values determine strict or relaxed coding
### current setting would be relaxed coding
chrom$perc.acro[chrom$perc.acro <= .1] <- 0
chrom$perc.acro[chrom$perc.acro >= .9] <- 0
chrom$perc.acro[chrom$perc.acro > 0] <- 1

### range should be larger than that observed in the
### data however if it should be as small as possible without
### impacting inference to speed computation
chroms <- datatoMatrix(chrom, range=c(3,49), hyper=T)
# remeber that this is making a matrix of probability of being in state 1
# and in this run of all mammals state 1 of 2 will be mixed

library(diversitree)

# set the MCMC chain length
iter <- 1000

### w is best set by doing a short run and getting some idea of the 
### confidence interval on these parameters
lk.mk <- make.mkn(tree[[1]], states=chroms, k=ncol(chroms), strict=F, control=list(method="ode"))

### This sets up the model described in the paper
con.lk.mk<-constrainMkn(data=chroms, lik=lk.mk, hyper=T, 
                        polyploidy=F, verbose=F, 
                        constrain=list(drop.demi=T, drop.poly=T))
temp <- mcmc(con.lk.mk, x.init=runif(6,0,10), w=1, nsteps=iter/10)
w <- diff(sapply(temp[1:6], quantile, c(.05, .95)))


### this will cycle through all 100 trees
results <- list()
for(i in 1:100){
  # make the initial likelihood function
  lk.mk <- make.mkn(tree[[i]], states = chroms, k = ncol(chroms), 
                    strict = F, control = list(method = "ode"))
  # constrain to a biologically realistic model of chrom evolution
  con.lk.mk<-constrainMkn(data = chroms, lik = lk.mk, hyper = T, 
                          polyploidy = F, verbose = F, 
                          constrain = list(drop.demi = T, drop.poly = T))
  # run the MCMC
  results[[i]] <- mcmc(con.lk.mk, x.init = colMeans(temp)[1:6], 
                       w = w, nsteps = iter)
}


