library(ape)
# read the trees
carn <- read.nexus("../data/proc.data/subclade.data/carn.nex")

# read the data
p.carn.chrom <- read.csv("../data/proc.data/subclade.data/carn.csv")

library(chromePlus)
####
#### state one is intermediate karyotype

range(p.carn.chrom[,2])
chroms.carn <- datatoMatrix(p.carn.chrom, range=c(14,41), hyper=T)


library(diversitree)
iter <- 50000

carn.results <-list()

# run in parallel
library(doMC)
registerDoMC(20)
x <- foreach(i=1:100) %dopar% {
  cat("\n starting tree", i)
  cat("\nCarnivores")
  lk.mk <- make.mkn(carn[[i]], states = chroms.carn, k = 56, strict = F, 
                    control = list(method = "ode"))
  con.lk.mk<-constrainMkn(data = chroms.carn, 
                          lik = lk.mk, 
                          hyper = T, 
                          polyploidy = F,
                          verbose = F, 
                          constrain = list(drop.demi = T, drop.poly = T))
  carn.results[[i]] <- diversitree::mcmc(con.lk.mk,print.every = 100,
                                         lower = rep(0,6),
                                         upper = rep(200,6),
                                         prior = make.prior.exponential(1),
                                         x.init = runif(6, 0, 5),
                                         w = 10, nsteps = iter)
}

save(file = "carnivore.50k.RData")



