library(ape)
# read the trees
prim <- read.nexus("../data/proc.data/subclade.data/prim.nex")
ceta <- read.nexus("../data/proc.data/subclade.data/cetart.nex")
chiro <- read.nexus("../data/proc.data/subclade.data/chiro.nex")
mars <- read.nexus("../data/proc.data/subclade.data/mars.nex")

# read the data
p.ceta.chrom <- read.csv("../data/proc.data/subclade.data/cetart.csv")
p.prim.chrom <- read.csv("../data/proc.data/subclade.data/prim.csv")
p.chiro.chrom <- read.csv("../data/proc.data/subclade.data/chiro.csv")
p.mars.chrom <- read.csv("../data/proc.data/subclade.data/mars.csv")

library(chromePlus)
####
#### state one is intermediate karyotype

range(p.ceta.chrom[,2])
chroms.ceta <- datatoMatrix(p.ceta.chrom, range=c(2,39), hyper=T)
range(p.prim.chrom[,2])
chroms.prim <- datatoMatrix(p.prim.chrom, range=c(8,42), hyper=T)
range(p.chiro.chrom[,2])
chroms.chiro <- datatoMatrix(p.chiro.chrom, range=c(5,33), hyper=T)
range(p.mars.chrom[,2])
chroms.mars <- datatoMatrix(p.mars.chrom, range=c(5,13), hyper=T)


library(diversitree)
iter <- 5000

prim.results <- list()
ceta.results <- list()

# analyze bats
lk.mk <- make.mkn(chiro, states=chroms.chiro, k=58, strict=F, control=list(method="ode"))
con.lk.mk.chiro<-constrainMkn(data=chroms.chiro, 
                        lik=lk.mk, 
                        hyper=T, 
                        polyploidy=F,
                        verbose=F, 
                        constrain=list(drop.demi=T, drop.poly=T))
chiro.results <- diversitree::mcmc(con.lk.mk.chiro,print.every = 250,
                           lower=rep(0,6),
                           upper=rep(200,6),
                           x.init=runif(min=0,max=5, 6),
                           w=c(11,15,2,6,2,3), nsteps=20000)

# analyze marsupials
lk.mk <- make.mkn(mars, states=chroms.mars, k=18, strict=F, control=list(method="ode"))
con.lk.mk.mars<-constrainMkn(data=chroms.mars, 
                        lik=lk.mk, 
                        hyper=T, 
                        polyploidy=F,
                        verbose=F, 
                        constrain=list(drop.demi=T, drop.poly=T))
mars.results <- diversitree::mcmc(con.lk.mk.mars,print.every = 250,
                          lower=rep(0,6),
                          upper=rep(200,6),
                          x.init=runif(min=0,max=5, 6),
                          w=c(11,15,2,6,2,3), nsteps=20000)



for(i in 1:100){
  cat("\n starting tree", i)
  cat("\nPrimates")
  # analyze primates
  lk.mk <- make.mkn(prim[[i]], states=chroms.prim, k=70, strict=F, control=list(method="ode"))
  con.lk.mk<-constrainMkn(data=chroms.prim, 
                          lik=lk.mk, 
                          hyper=T, 
                          polyploidy=F,
                          verbose=F, 
                          constrain=list(drop.demi=T, drop.poly=T))
  prim.results[[i]] <- diversitree::mcmc(con.lk.mk, print.every = 100,
                            lower=rep(0,6),
                            upper=rep(200,6),
                            x.init=runif(min=0,max=5, 6), 
                     w=c(10,10,1,1,1,1), nsteps=iter)
  cat("\nCetrtiodactyla")
  
  # analyze cetartiodactyla
  lk.mk <- make.mkn(ceta[[i]], states=chroms.ceta, k=76, strict=F, control=list(method="ode"))
  con.lk.mk<-constrainMkn(data=chroms.ceta, 
                          lik=lk.mk, 
                          hyper=T, 
                          polyploidy=F,
                          verbose=F, 
                          constrain=list(drop.demi=T, drop.poly=T))
  ceta.results[[i]] <- diversitree::mcmc(con.lk.mk, print.every = 100,
                            lower=rep(0,6),
                            upper=rep(200,6),
                            x.init=runif(min=0,max=5, 6), 
                            w=c(10,10,1,1,1,1), nsteps=iter)
}




