# Heath Blackmon
# August 18, 2017
# coleoguy@gmail.com

# this script is used to perform simulation work
# described in the chromPlus paper
# utility functions
UnitScale <- function(x){
  z <- max(branching.times(x))
  x$edge.length <- x$edge.length/z
  return(x)
}


library(diversitree)
# number of taxa that we will use
taxa <- c(50,100,200,500)

# this will hold all trees
all.trees <- vector(length=4,mode="list")
names(all.trees) <- paste(taxa,"Taxa",sep="")

# simulate trees
for(i in 1:4){
  all.trees[[i]] <- trees(pars = c(3,1),
                          type = "bd",
                          n = 100,
                          max.taxa = taxa[i],
                          include.extinct = F)
}

# rescale trees
for(i in 1:4){
  for(ii in 1:100){
    all.trees[[i]][[ii]] <- UnitScale(all.trees[[i]][[ii]])
  }
}

# simulation of tip data
library(chromePlus)

# rate multipliers
rmult <- c(1, 2, 5, 10)


# containers to hold data
bin.models <-  vector(length=4,mode="list")
names(bin.models) <- c("mult1", "mult2", "mult5", "mult10")
chrom.models <- bin.models
record.tries <- vector()
# rate classes
for(i in 1:4){
  total.counter <- 0
  # containers to hold data
  all.binary <-  vector(length=4,mode="list")
  names(all.binary) <- paste("Taxa",taxa,sep="")
  all.chroms <- all.binary
  # tree sizes
  for(ii in 1:4){
    # containers to hold data
    bin.sim <- chr.sim <- list()
    # individual trees
    for(iii in 1:100){
      # condition on at least 5 in each state
      check <- F
      while(check==F){
        total.counter <- total.counter + 1
        z <- simChrom(all.trees[[ii]][[iii]], 
                      pars=c(.75, rmult[i]*.75, .75, rmult[i]*.75,
                             0,   0,  0,  0, .5,  .5, 10,  0),
                      limits = c(1, 100), model = "ChromTrait")
        if(sum(z$binary.state) > 5 &
           sum(z$binary.state) < (length(z$binary.state)-5)){
          check <- T
        }
      }
      bin.sim[[iii]] <- z$binary.state
      chr.sim[[iii]] <- z$chrom.num
    }
    all.binary[[ii]] <- bin.sim 
    all.chroms[[ii]] <- chr.sim
  }
  record.tries[i] <- total.counter
  bin.models[[i]] <- all.binary
  chrom.models[[i]] <- all.chroms
}


hist(chrom.models[[4]][[4]][[1]])
hist(chrom.models[[1]][[4]][[1]])


# all simulated data is done now we can clean up and then analyze
foo <- ls()
foo <- ls()
foo <- foo[c(-3,-4,-8)]     
rm(list=foo)

GetOverlap <- function(x, burnin){
  library(coda)
  topnum <- nrow(x) - burnin
  z <- vector()
  for(i in 1:topnum){
    j <- i + burnin
    z[i] <- ((x[j, 2] + x[j, 3]) / 2) -
      ((x[j, 4] + x[j, 5]) / 2)
  }
  z <- HPDinterval(as.mcmc(z), prob=0.95)
  return(z)
}

# set up results container
results.L <- matrix(,100,16)
colnames(results.L) <- c("rmult1.taxa50",
                       "rmult1.taxa100",
                       "rmult1.taxa200",
                       "rmult1.taxa500",
                       "rmult2.taxa50",
                       "rmult2.taxa100",
                       "rmult2.taxa200",
                       "rmult2.taxa500",
                       "rmult5.taxa50",
                       "rmult5.taxa100",
                       "rmult5.taxa200",
                       "rmult5.taxa500",
                       "rmult10.taxa50",
                       "rmult10.taxa100",
                       "rmult10.taxa200",
                       "rmult10.taxa500")
results.H <- results.L

counter <- 1
for(i in 1:4){ #rmultipliers
  cat(paste("rate multiplier", i))
  for(ii in 1:4){ # taxa numbers
    cat(paste("\n", names(chrom.models[[i]])[[ii]], sep=""))
    for(iii in 1:100){ # tree replicates
      cat(paste("\ntree:", iii))
      cdat <- data.frame(names(chrom.models[[i]][[ii]][[iii]]),
                         chrom.models[[i]][[ii]][[iii]], 
                         bin.models[[i]][[ii]][[iii]])
      colnames(cdat) <- c("taxa", "haploid", "bin")
      z <- range(cdat[,2])
      z[1] <- z[1] - 1
      z[2] <- z[2] + 1
      cmat <- datatoMatrix(cdat, range=z, hyper=T)
      lk.mk <- make.mkn(all.trees[[ii]][[iii]], states=cmat, k=ncol(cmat), 
                        strict=F, control=list(method="ode"))
      con.lk <-constrainMkn(data=cmat, 
                            lik=lk.mk, 
                            hyper=T, 
                            polyploidy=F,
                            verbose=F, 
                            constrain=list(drop.demi=T, drop.poly=T))
      x <- diversitree::mcmc(con.lk, print.every = 250,
                lower=rep(0,6),
                upper=rep(200,6),
                x.init=runif(min=0,max=5, 6), 
                w=1, nsteps=1000)
      res <- GetOverlap(x, burnin=500)
      results.L[iii, counter] <- res[1]
      results.H[iii, counter] <- res[2]
    }
  counter <- counter + 1
  }
}



## This is the script for the BiSSE simulations

UnitScale <- function(x){
  z <- max(branching.times(x))
  x$edge.length <- x$edge.length/z
  return(x)
}

GetOverlap <- function(x, burnin){
  library(coda)
  topnum <- nrow(x) - burnin
  z <- vector()
  for(i in 1:topnum){
    j <- i + burnin
    z[i] <- ((x[j, 2] + x[j, 3]) / 2) -
      ((x[j, 4] + x[j, 5]) / 2)
  }
  z <- HPDinterval(as.mcmc(z), prob=0.95)
  return(z)
}

# this chunk will simulate the trees with state 1 (of 0 and 1)
# having a speciation rate twice that of state 0.

library(diversitree)
pars <- c(3, 6, 1, 1, .5, .5)
trees <- list()
for(i in 1:100){
  foo <- NULL
  while(is.null(foo)){
    foo <- tree.bisse(pars, max.taxa=200, x0=0)
  }
  print(i)
  trees[[i]] <- UnitScale(foo)
}

# this chunk will simulate chromosome number for all the tips
# with no relationship to binary state
for(i in 1:100){
  trees[[i]][[11]] <- sim.character(trees[[i]], model="meristic", x0=10, pars=c(20, 2, 2))
  names(trees[[i]])[[11]] <- "chrome.num"
  names(trees[[i]])[[4]] <- "bi.state"
}

# now we need to evaluate these datasets with and without accounting for
# SSE

library(chromePlus)
x.w <- x.wo <- list()
for(i in 1:100){
  print(paste("working on tree", i))
  cdat <- data.frame(trees[[i]]$tip.label,
                     as.numeric(trees[[i]]$chrome.num),
                     as.numeric(trees[[i]]$bi.state))
  colnames(cdat) <- c("taxa", "haploid", "bin")
  z <- range(cdat[,2])
  z[1] <- z[1] - 1
  z[2] <- z[2] + 1
  cmat <- datatoMatrix(cdat, range=z, hyper=T)
  lk.mk <- make.musse(trees[[i]], states=cmat, k=ncol(cmat), 
                      strict=F, control=list(method="ode"))
  wo.sse <-constrainMuSSE(data=cmat, 
                          lik=lk.mk, 
                          hidden=T, polyploidy=F,
                          verbose=F, 
                          s.lambda = T, s.mu = T,
                          constrain=list(drop.demi=T, drop.poly=T))
  w.sse <-constrainMuSSE(data=cmat, 
                         lik=lk.mk, 
                         hidden=T, polyploidy=F,
                         verbose=F, 
                         s.lambda = F, s.mu = F,
                         constrain=list(drop.demi=T, drop.poly=T))
  x.w[[i]] <- diversitree::mcmc(w.sse, print.every = 25,
                                lower=rep(0,10),
                                upper=rep(20,10),
                                x.init=runif(min=0,max=5, 10), 
                                w=1, nsteps=1000)
  x.wo[[i]] <- diversitree::mcmc(wo.sse, print.every = 25,
                                 lower=rep(0,8),
                                 upper=rep(20,8),
                                 x.init=runif(min=0,max=5, 8), 
                                 w=1, nsteps=1000)
}


results <- as.data.frame(matrix(,100,11))
colnames(results) <- c("tree", "deltar.wo", "deltar.w", 
                       "hpd.95.wo.l", "hpd.95.wo.h", 
                       "hpd.95.w.l", "hpd.95.w.h",
                       "trans.wo.12", "trans.wo.21", 
                       "trans.w.12", "trans.w.21")
# x mean rate in 1 wo
# y mean rate in 2 wo
# r mean rate in 1 w
# s mean rate in 2 w
burnin <- 750
for(i in 1:100){
  x <- mean(c(x.wo[[i]]$asc1[burnin:1000], x.wo[[i]]$desc1[burnin:1000]))
  y <- mean(c(x.wo[[i]]$asc2[burnin:1000], x.wo[[i]]$desc2[burnin:1000]))
  r <- mean(c(x.w[[i]]$asc1[burnin:1000], x.w[[i]]$desc1[burnin:1000]))
  s <- mean(c(x.w[[i]]$asc2[burnin:1000], x.w[[i]]$desc2[burnin:1000]))
  results[i,1] <- i
  results[i,2] <- x-y
  results[i,3] <- r-s
  results[i,4:5] <- as.numeric(GetOverlap(x.wo[[i]], burnin=500))
  results[i,6:7] <- as.numeric(GetOverlap(x.w[[i]], burnin=500))
  results[i,8] <- mean(x.wo[[i]]$tran12[burnin:1000])
  results[i,9] <- mean(x.wo[[i]]$tran21[burnin:1000])
  results[i,10] <- mean(x.w[[i]]$tran12[burnin:1000])
  results[i,11] <- mean(x.w[[i]]$tran21[burnin:1000])
}


# get range of chromosome number in lowest rate smallest trees
sm.chroms <- all.chroms[[1]]
lg.chroms <- all.chroms[[4]]
chrom.rngs <- chrom.rngl <- c()
for(i in 1:100){
  chrom.rngs[i] <- range(sm.chroms[[i]])[2] - range(sm.chroms[[i]])[1]
  chrom.rngl[i] <- range(lg.chroms[[i]])[2] - range(lg.chroms[[i]])[1]
}
median(chrom.rngs)
median(chrom.rngl)


bin.models[[1]][[1]]

