# analysis using the whale tree
library(diversitree)
library(chromePlus)
tree <- read.tree("../data/raw.data/trees/whales.tre")
library(phytools)
# due to numerical precision tree is judged non-ultrametric
tree <- force.ultrametric(tree)


# High Rates
pars <- c(.75, .75, .75, .75,
          0,   0,  0,  0,
          .5, .5, 20, 1)
results <- list()
library(doMC)
registerDoMC(7)
for(i in 100:100){
  result <- simChrom(tree = tree, pars = pars, 
                     limits = c(2, 40), 
                     model = "ChromPlus")
  dat.chrom <- data.frame(names(result$chrom.num),
                          result$chrom.num,
                          result$binary.state)
  x <- range(dat.chrom[,2])
  chroms.cet <- datatoMatrix(dat.chrom, 
                             range = c(x[1]-1, x[2]+1), 
                             hyper = T)
  lk.mk <- make.musse(tree, 
                      states = chroms.cet, 
                      k = ncol(chroms.cet), 
                      strict = F, 
                      control = list(method="ode"))
  
  con.lk <-constrainMuSSE(data = chroms.cet, 
                          lik = lk.mk, 
                          hyper = T, 
                          polyploidy = F,
                          verbose = F, 
                          s.lambda = F,
                          s.mu = F,
                          constrain = list(drop.demi = T, 
                                           drop.poly = T))
  results[[i]] <- diversitree::mcmc(con.lk,
                                    lower=rep(0,10),
                                    upper=rep(5,10),
                                    x.init=runif(min=0,max=1, 10),
                                    w=1, nsteps=400,
                                    print.every=50)
}

low.rates <- results

# Rerun with Low Rates
pars <- c(.1, .1, .1, .1,
          0,   0,  0,  0,
          .1, .1, 20, 1)

